==> drupalmoduleupgrader-8.x-0.6.tar.gz: drupalmoduleupgrader/src/Plugin/DMU/Converter/Tests.php <==
<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\Filter\ContainsLogicFilter;
use Pharborist\ClassMemberNode;
use Pharborist\ClassMethodCallNode;
use Pharborist\ClassNode;
use Pharborist\DocCommentNode;
use Pharborist\Filter;
use Pharborist\Parser;
use Pharborist\StringNode;
use Pharborist\TopNode;
use Pharborist\WhitespaceNode;

/**
 * @Converter(
 *  id = "tests",
 *  description = @Translation("Modifies test classes.")
 * )
 */
class Tests extends ConverterBase {

  private $target;

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    $this->target = $target;

    $target->getIndexer('web_test')->getAll()->each([ $this, 'convertWeb' ]);
    $target->getIndexer('unit_test')->getAll()->each([ $this, 'convertUnit' ]);

    $target
      ->getIndexer('test')
      ->getAll()
      ->filter(function(ClassNode $class) {
        return $class->getExtends() == 'AJAXTestCase';
      })
      ->each([ $this, 'convertAjax' ]);
  }

  /**
   * Converts a single web test.
   *
   * @param \Pharborist\ClassNode $test
   */
  public function convertWeb(ClassNode $test) {
    $test->setExtends('\Drupal\simpletest\WebTestBase');
    $this->convertInfo($test);
    $this->setModules($test);
    $this->setProfile($test);
    $this->move($test);
  }

  /**
   * Converts the test's getInfo() method to an annotation.
   *
   * @param \Pharborist\ClassNode $test
   */
  private function convertInfo(ClassNode $test) {
    $info = $this->extractInfo($test);

    if ($info) {
      $comment = '';
      $comment .= $info['description'] . "\n\n";
      $comment .= '@group ' . $this->target->id();

      if (isset($info['dependencies'])) {
        $comment .= "\n";
        foreach ($info['dependencies'] as $module) {
          $comment .= '@requires module . ' . $module . "\n";
        }
      }

      $test->setDocComment(DocCommentNode::create($comment));
    }
    else {
      drush_log('Could not get info for test ' . $test->getName() . '.', 'error');
    }
  }

  /**
   * Extracts the return value of the test's getInfo() method, if there's no
   * logic in the method.
   *
   * @param \Pharborist\ClassNode $test
   *
   * @return array|NULL
   */
  private function extractInfo(ClassNode $test) {
    if ($test->hasMethod('getInfo')) {
      $info = $test->getMethod('getInfo');

      if (! $info->is(new ContainsLogicFilter())) {
        return eval($info->getBody()->getText());
      }
    }
  }

  /**
   * Sets the test's $modules property.
   *
   * @param \Pharborist\ClassNode $test
   */
  private function setModules(ClassNode $test) {
    $modules = $this->extractModules($test);
    if ($modules) {
      // @todo Use ClassNode::createProperty() when #124 lands in Pharborist
      $property = Parser::parseSnippet('class Foo { public static $modules = ["' . implode('", "', $modules) . '"]; }')
        ->getBody()
        ->firstChild()
        ->remove();
      $test->appendProperty($property);
    }
  }

  /**
   * Extracts every module required by a web test by scanning its calls
   * to parent::setUp().
   *
   * @param \Pharborist\ClassNode $test
   *
   * @return string[]
   *  Array of modules set up by this module.
   */
  private function extractModules(ClassNode $test) {
    $modules = [];

    $test
      ->find(Filter::isClassMethodCall('parent', 'setUp'))
      ->filter(function(ClassMethodCallNode $call) {
        return (sizeof($call->getArguments()) > 0);
      })
      ->each(function(ClassMethodCallNode $call) use (&$modules) {
        foreach ($call->getArguments() as $argument) {
          if ($argument instanceof StringNode) {
            $modules[] = $argument->toValue();
          }
        }

        $call->clearArguments();
      });

    return array_unique($modules);
  }

  /**
   * Sets the test's $profile property.
   *
   * @param \Pharborist\ClassNode $test
   */
  private function setProfile(ClassNode $test) {
    if (! $test->hasProperty('profile')) {
      $test->appendProperty(ClassMemberNode::create('profile', StringNode::create("'standard'"), 'protected'));
    }
  }

  public function move(ClassNode $test) {
    $ns = 'Drupal\\' . $this->target->id() . '\\Tests';
    TopNode::create($ns)->getNamespace($ns)->append($test->remove());
    WhitespaceNode::create("\n\n")->insertBefore($test);

    $this->writeClass($this->target, $test);
  }

  /**
   * Converts a single unit test.
   *
   * @param \Pharborist\ClassNode $test
   */
  public function convertUnit(ClassNode $test) {
    $test->setExtends('\Drupal\Tests\UnitTestCase');

    $comment_text = <<<END
@FIXME
Unit tests are now written for the PHPUnit framework. You will need to refactor this
test in order for it to work properly.
END;
    $comment = DocCommentNode::create($comment_text);
    $test->setDocComment($comment);

    $ns = 'Drupal\Tests\\' . $this->target->id() . '\Unit';
    $doc = TopNode::create($ns)->getNamespace($ns)->append($test->remove());
    WhitespaceNode::create("\n\n")->insertBefore($test);

    $this->write($this->target, 'tests/src/Unit/' . $test->getName() . '.php', "<?php\n\n$doc");
  }

  /**
   * Converts a single Ajax test.
   *
   * @param \Pharborist\ClassNode $test
   */
  public function convertAjax(ClassNode $test) {
    $test->setExtends('\Drupal\system\Tests\Ajax\AjaxTestBase');
    $this->setModules($test);
    $this->move($test);
  }

}

==> drupalmoduleupgrader-8.x-0.7.tar.gz: drupalmoduleupgrader/src/Plugin/DMU/Converter/Tests.php <==
<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\Filter\ContainsLogicFilter;
use Pharborist\Objects\ClassMemberNode;
use Pharborist\Objects\ClassMethodCallNode;
use Pharborist\Objects\ClassNode;
use Pharborist\DocCommentNode;
use Pharborist\Filter;
use Pharborist\Parser;
use Pharborist\RootNode;
use Pharborist\Types\StringNode;
use Pharborist\WhitespaceNode;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "tests",
 *  description = @Translation("Modifies test classes.")
 * )
 */
class Tests extends ConverterBase implements ContainerFactoryPluginInterface {

  private $target;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  private $log;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelInterface $log) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->log = $log;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')->get('drupalmoduleupgrader')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isExecutable(TargetInterface $target) {
    return ($target->getIndexer('test')->count() || $target->getIndexer('web_test')->count());
  }

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    $this->target = $target;

    $target->getIndexer('web_test')->getAll()->each([ $this, 'convertWeb' ]);

    $target
      ->getIndexer('test')
      ->getAll()
      ->filter(function(ClassNode $class) {
        return $class->getExtends() == 'AJAXTestCase';
      })
      ->each([ $this, 'convertAjax' ]);
  }

  /**
   * Converts a single web test.
   *
   * @param \Pharborist\ClassNode $test
   */
  public function convertWeb(ClassNode $test) {
    $test->setExtends('\Drupal\simpletest\WebTestBase');
    $this->convertInfo($test);
    $this->setModules($test);
    $this->setProfile($test);
    $this->move($test);
  }

  /**
   * Converts the test's getInfo() method to an annotation.
   *
   * @param \Pharborist\ClassNode $test
   */
  private function convertInfo(ClassNode $test) {
    $info = $this->extractInfo($test);

    if ($info) {
      $comment = '';
      $comment .= $info['description'] . "\n\n";
      $comment .= '@group ' . $this->target->id();

      if (isset($info['dependencies'])) {
        $comment .= "\n";
        foreach ($info['dependencies'] as $module) {
          $comment .= '@requires module . ' . $module . "\n";
        }
      }

      $test->setDocComment(DocCommentNode::create($comment));
    }
    else {
      $this->log->error('Could not get info for test {class}.', [
        'class' => $test->getName(),
      ]);
    }
  }

  /**
   * Extracts the return value of the test's getInfo() method, if there's no
   * logic in the method.
   *
   * @param \Pharborist\ClassNode $test
   *
   * @return array|NULL
   */
  private function extractInfo(ClassNode $test) {
    if ($test->hasMethod('getInfo')) {
      $info = $test->getMethod('getInfo');

      if (! $info->is(new ContainsLogicFilter())) {
        return eval($info->getBody()->getText());
      }
    }
  }

  /**
   * Sets the test's $modules property.
   *
   * @param \Pharborist\ClassNode $test
   */
  private function setModules(ClassNode $test) {
    $modules = $this->extractModules($test);
    if ($modules) {
      // @todo Use ClassNode::createProperty() when #124 lands in Pharborist
      $property = Parser::parseSnippet('class Foo { public static $modules = ["' . implode('", "', $modules) . '"]; }')
        ->getBody()
        ->firstChild()
        ->remove();
      $test->appendProperty($property);
    }
  }

  /**
   * Extracts every module required by a web test by scanning its calls
   * to parent::setUp().
   *
   * @param \Pharborist\ClassNode $test
   *
   * @return string[]
   *  Array of modules set up by this module.
   */
  private function extractModules(ClassNode $test) {
    $modules = [];

    $test
      ->find(Filter::isClassMethodCall('parent', 'setUp'))
      ->filter(function(ClassMethodCallNode $call) {
        return (sizeof($call->getArguments()) > 0);
      })
      ->each(function(ClassMethodCallNode $call) use (&$modules) {
        foreach ($call->getArguments() as $argument) {
          if ($argument instanceof StringNode) {
            $modules[] = $argument->toValue();
          }
        }

        $call->clearArguments();
      });

    return array_unique($modules);
  }

  /**
   * Sets the test's $profile property.
   *
   * @param \Pharborist\ClassNode $test
   */
  private function setProfile(ClassNode $test) {
    if (! $test->hasProperty('profile')) {
      $test->appendProperty(ClassMemberNode::create('profile', StringNode::create("'standard'"), 'protected'));
    }
  }

  public function move(ClassNode $test) {
    $ns = 'Drupal\\' . $this->target->id() . '\\Tests';
    RootNode::create($ns)->getNamespace($ns)->append($test->remove());
    WhitespaceNode::create("\n\n")->insertBefore($test);

    $this->writeClass($this->target, $test);
  }

  /**
   * Converts a single Ajax test.
   *
   * @param \Pharborist\ClassNode $test
   */
  public function convertAjax(ClassNode $test) {
    $test->setExtends('\Drupal\system\Tests\Ajax\AjaxTestBase');
    $this->setModules($test);
    $this->move($test);
  }

}
